// start an instance of the timer library
var easy_timer_obj = new easytimer.Timer();

var core = {

	initialized: false,
	timer_active: false,
	mode: "",
	animation_time: 1000,

	// because of how the chrome.storage API works, you'll need to nest each instance where you grab data from the storage API within the chrome.storage.sync.get CALLBACK FUNCTION
	// due to this, I'm creating a simple helper method that will take the data value from "result.blocklist" and format it properly if it's null or undefined
	get_blocklist: function(callback = function(){}) {
		
		var format_blocklist = function(blocklist) {

			if (typeof blocklist === "object") {
				return blocklist;
			}

			return [];

		};
		
		chrome.storage.sync.get('blocklist', function(result) {
			blocklist = format_blocklist(result.blocklist);
			callback();
		});

	},

	elements: {

		main: {
			status: $("#status"), // the container element that wraps around the status text ("Active", "Paused", "On Until 5:15PM") and the time_remaining (" 11:58" [countdown])
			status_text: $("#status #status-text"), // Active, Paused, On Until 5:15PM
			master_switch: $("input.master-switch")
		},

		master_switch: {
			master_resume_btn: $("#master-resume-btn")
		},

		timer: {
			pause_btns: $("#pause-btns"), // the MULTIPLE pause btns, so pause_btn AND turn_off_btn
			turn_off_btn: $("#turn-off-btn"),
			pause_btn: $("#pause-btn"), // the main "Pause for 15 Minutes" timer
			control_btns: $("#control-btns"), // the buttons that come up AFTER a timer is set, pause and resume
			time_remaining: $("#status #time-remaining"), // 11:58... 11:57... 11:56...
			add_btn: $("#add-btn"), // "+15 minutes" button when timer is active
			resume_btn: $("#resume-btn") // "Resume" button when timer is active
		},

		schedule: {
			edit_my_schedule_btn: $("#edit-my-schedule-btn"), // the button that comes up when schedule mode is active and a timer is NOT set
			schedule_mode_switch: $("#schedule-mode-switch"),
			schedule_save_btn: $("#schedule-save-button")
		}

	},

	helpers: {

		animator: function(prev_view, new_view) {

			// will be used to determine "NOTs", basically - so that you can hide everything EXCEPT 'schedule', or whatever
			var elements = {
				schedule: core.elements.schedule.edit_my_schedule_btn,
				paused: core.elements.timer.control_btns,
				default: core.elements.timer.pause_btns,
				master_switch_off: core.elements.master_switch.master_resume_btn
			};

			var keys = $.map(elements, function(element,index) {return index}),
				unused = keys.filter(function(key){return key != prev_view && key != new_view});

			unused.forEach(function(key){
				elements[key].fadeOut(0);
			});

			// unused.fadeOut();

			if (core.initialized == false || prev_view == '' || prev_view == undefined) {

				elements[new_view].fadeOut(0, function(){					
					elements[new_view].fadeIn(0);
				});

			} else {

				elements[new_view].fadeOut(0, function(){					
					elements[prev_view].fadeOut(core.animation_time, function(){
						elements[new_view].fadeIn(core.animation_time);
					});
				});
			
			};

		} 

	},

	schedule: {
		
		// check to see if time is within schedule
		is_scheduled: function(schedule) {

		    var days = {0: "sun", 1: "mon", 2: "tue", 3: "wed", 4: "thu", 5: "fri", 6: "sat"};

		    var current_date = new Date(),
		        day = days[current_date.getDay()];

		    var today = new Date(),
		        time = today.getHours() + ":" + today.getMinutes();

		    // the day of the week is active
		    if (schedule.days[day] == true) {

		        // the time of day is active
		        if ( Date.parse(schedule.times.from) < current_date && Date.parse(schedule.times.to) > current_date ) {
		            return true;
		        };

		    };

		    return false;

		}

	},

	view: {

		master_switch_off: function() {

			// remove the "Edit my schedule button" and remove any easytimer stuff
			core.elements.schedule.edit_my_schedule_btn.fadeOut(0);
			easy_timer_obj.stop();
			core.elements.timer.time_remaining.text("");

			// "Active" in green text
			core.elements.main.status.addClass("red-text").removeClass("green-text");
			core.elements.main.status_text.text("One Deep Breath is off");


			// core.elements.timer.pause_btn.fadeOut(0);			
			// core.elements.timer.control_btns.fadeOut(0);
			// transitions/animations
			// new, fresh view - not initialized, so no transitions
			// core.helpers.animator(core.mode, 'default');

			
			core.helpers.animator(core.mode, 'master_switch_off');
			core.mode = 'master_switch_off';

		},

		paused: function(master_timer_value) {

			// if the timer is NOT already initialized, then DON't do a stupid ass transition
			// if (core.initialized == false) {

			// 	core.elements.timer.pause_btns.fadeOut(0, function(){					
			// 		core.elements.timer.control_btns.fadeIn(0);
			// 	});

			// }

			// // if the timer IS initialized, and the user just clicked "Pause for 15 mins" THEN do a cool transition
			// if (core.initialized == true && timer_active == false) {
				
			// 	core.elements.timer.control_btns.fadeOut(0, function(){					
			// 		core.elements.timer.pause_btns.fadeOut(core.animation_time, function(){
			// 			core.elements.timer.control_btns.fadeIn(core.animation_time);
			// 		});
			// 	});

			// }

			// all related to settings up the timer
			var time = Date.parse(master_timer_value), // this is the time stringw which is equal to the time the user clicked the pause button + 15 minutes (or 30, or 45, or whatever)
				current_time = new Date,
				time_remaining = new TimeSpan(time - current_time), // get the difference between the current time and the time the timer should be off
				timer_format = (time_remaining.hours > 0) ? ['hours', 'minutes'] : ['minutes', 'seconds']; // if there is more than an hour remaining, then the format should be "HOURS:MINUTES", otherwise "MINS:SECONDS"

			// this'll initialize the timer object and start the actual countdown
			easy_timer_obj.stop();
			easy_timer_obj.start({countdown: true, startValues: {hours: time_remaining.hours, minutes: time_remaining.minutes, seconds: time_remaining.seconds}});
			easy_timer_obj.removeEventListener('secondsUpdated');
			easy_timer_obj.removeEventListener('targetAchieved');
			
			// update basic view elements
			core.elements.main.status.removeClass("green-text").addClass("red-text"); // if the timer is paused the timer should be red
			core.elements.main.status_text.text("Paused for"); // the actual text shuold say, "Paused for "...
			core.elements.timer.time_remaining.html(easy_timer_obj.getTimeValues().toString(timer_format)); // set the value of this object to whatever time is remaining
			
			// update the view every second
			easy_timer_obj.addEventListener('secondsUpdated', function (e) {
			    core.elements.timer.time_remaining.html(easy_timer_obj.getTimeValues().toString(timer_format));
			});
			
			// when the timer is over, update the view
			easy_timer_obj.addEventListener('targetAchieved', function (e) {
			    setTimeout(function(){
			    	timer.update_view();
			    }, 1000);
			});

			// this will be used to determine which transitions need to occur (should we fade out buttons, or just make it happen without transitions [ie: is it initialized?])
			timer_active = true;
			core.mode = 'paused';
			core.helpers.animator(core.mode, 'paused');

		},

		schedule_mode: function(status, schedule) {

			// remove any of the pause/timer buttons and have the "Edit my schedule" button be available
			core.elements.timer.control_btns.fadeOut(0);
			core.elements.timer.pause_btns.fadeOut(0);
			core.elements.schedule.edit_my_schedule_btn.fadeIn(0);

			// stop easytimer in case it's running - this method WILL NOT be run if a timer is active, so this should be fine - basically, this shouldn't do anything
			easy_timer_obj.stop();
			core.elements.timer.time_remaining.text("");

			// if schedule is active, then have "On Until #{TIME}"
			if (status.schedule_active) {
				core.elements.main.status.addClass("green-text").removeClass("red-text");
				core.elements.main.status_text.text("On Until " + schedule.times.to);							
			} 
			else // schedule is inactive, then have "Off Until #{TIME}"
			{	
			
				core.elements.main.status.removeClass("green-text").addClass("red-text");
				
				// this will contain the day in which the timer will be active again IF it won't be active today...
				// so, if today is Monday, and the timer won't be active until 5:00AM Wednesday, then return
				// "Wed"
				// otherwise, don't return anything
				var optional_days_text = function() {

					var days = {0: "sun", 1: "mon", 2: "tue", 3: "wed", 4: "thu", 5: "fri", 6: "sat"};

					var current_date = new Date(),
					    day = days[current_date.getDay()];

					// if it's not active today, find the next day it will be active and have that appear in the UI 
					if (schedule.days[day] == false) {

						// the integer value of the day - loop through "for" until you find the next active day... if you can't find it, then turn schedule off
						var day_int = current_date.getDay(),
							next_active_day;

						for (var i = 0; i <= 6; i++) {
							
							var v = day_int + i;

							if (v > 6) {
								v = v - 7;
							}

							if (schedule.days[days[v]] == true) {
								next_active_day = days[v];
								break;	
							}

						}

						if (next_active_day != undefined) {
							return " " + next_active_day.replace(/\w\S*/g, function(txt){return txt.charAt(0).toUpperCase() + txt.substr(1).toLowerCase();});
						} else {
							return "";
						}

					}

					return "";

				}();

				core.elements.main.status_text.text("Off Until " + schedule.times.from.replace(" ", "") + optional_days_text);
			
			};

			core.helpers.animator(core.mode, 'schedule');
			core.mode = 'schedule';

		},

		default: function(master_switch) {

			// remove the "Edit my schedule button" and remove any easytimer stuff
			core.elements.schedule.edit_my_schedule_btn.fadeOut(0);
			easy_timer_obj.stop();
			core.elements.timer.time_remaining.text("");

			// "Active" in green text
			core.elements.main.status.addClass("green-text").removeClass("red-text");
			core.elements.main.status_text.text("Active");

			if (master_switch == false) {
				core.elements.main.status_text.text("");				
			}

			// transitions/animations
			// new, fresh view - not initialized, so no transitions
			core.helpers.animator(core.mode, 'default');

			core.mode = 'default';

		}

	},

	controller: {

		update_view: function() {

			// get the time from chrome to compare to
			chrome.storage.sync.get(['master_timer', 'master_switch', 'schedule_mode_master_switch', 'schedules'], function(result) {

				var schedule = result.schedules[0],
					at_least_one_day_active = ($.map(schedule.days, function(v, k) { return v; }).filter(function(v){return v == true}).length > 0) // this will be used to determine if the user has selected AT LEASST one day of the week for the schedule;

				// this will store user information about whether or not schedule mode is on, and whether or not schedules are currently active
				var status = {
					paused: core.timer.is_paused(result.master_timer),
					schedule_on: (result.schedule_mode_master_switch == 'true' && at_least_one_day_active) ? true : false,
					schedule_active: false,
					master_switch: (result.master_switch == 'true')
				};

				core.elements.main.master_switch.prop("checked", true);
				if (status.master_switch == false) {

					if (core.initialized == false) {
						core.elements.main.master_switch.prop("checked", false);
					}

					core.view.master_switch_off();
					core.initialized = true;
					timer_active = false;
					return false;
				
				}

				// check to see if schedules are active (fall within user set times)
				if (status.schedule_on) {
					status.schedule_active = core.schedule.is_scheduled(result.schedules[0]);
				};

				// IF THE CURRENT TIME IS VALID, then IT SHOULD be PAUSED
				if (status.paused) {

					core.view.paused(result.master_timer);
				
				} else {

					// if schedule mode is turned on
					if (status.schedule_on) {

						core.view.schedule_mode(status, schedule);

					}
					  else //if schedule mode is turned off and main timer is not active 
					{

						core.view.default(result.master_switch == 'true');
						
					}


					timer_active = false;
					
				}

				core.initialized = true;

			});

		}

	},

	interaction: {

		timer: function() {
			
			// on the click of the resume button, unpause the timer
			core.elements.timer.resume_btn.on("click", function(){
				core.timer.unpause();
			});

			// on the click of the "turn off" button, click the master switch so that it's event fires and we don't have to rewrite that code
			core.elements.timer.turn_off_btn.on("click", function(){
				core.elements.main.master_switch.click();
			});

			// on the click of the pause button OR the "+15 Mins" button, add more time
			$([core.elements.timer.add_btn, core.elements.timer.pause_btn]).map(function() {return this.toArray();}).on("click", function(){
				core.timer.add_time(15);
			});

		},

		schedule: function() {

			// update the view if the user turns on the schedule or saves it
			$([core.elements.schedule.schedule_mode_switch, core.elements.schedule.schedule_save_btn]).map(function() {return this.toArray();}).on("click", function(){

				if (core.elements.main.master_switch.prop("checked") == false && $(this).prop("checked") != false) {
					core.elements.main.master_switch.click();
				} else {
					core.controller.update_view();
				}
			

			});

			// on the click of "edit my schedule" button...
			// core.elements.schedule.edit_my_schedule_btn.on("click", function() {

			// 	// if it's a popup, then get a new URL to pull up the options page with edit schedule selected, this will make it so the "Edit Schedule" portion of the dashboard automatically appears
			// 	if ($("body").hasClass("popup")) {
				
					
				
			// 	}

			// });

		},

		master_switch: function() {

			core.elements.master_switch.master_resume_btn.on("click", function() {
				$("body#zen-blocker-body input.master-switch").click();				
			});

			$("body#zen-blocker-body input.master-switch").on("change", function(){

				// console.log($("input#master-switch").prop("checked"));
				var val = $(this).prop("checked");

				// TODO
				// there is a bug right now where if you update the popup master-switch the options won't update
				// that's because the popup is a separate HTML document that can't access the options.html through jQuery
				// I could send a message, but I don't feel like doing that right now, so I won't
				if ( $(this).attr("id") == "popup" ) {

				}

				// make sure all master-switches that are currently active also update their UI
				$("body#zen-blocker-body input.master-switch").prop("checked", val);				

				chrome.storage.sync.set({master_switch: val.toString()}, function() {

					if (val == true) {
						core.controller.update_view();
					} else {
						core.view.master_switch_off();
					}

					chrome.runtime.sendMessage({request: 'update_settings'});

				});
			
			});

		},

		initialize: function() {
			core.interaction.timer();
			core.interaction.schedule();
			core.interaction.master_switch();
		}

	},

	timer: {

		is_paused: function(master_timer_time) {

			// get the current time to base the timer off of
			var current_time = new Date,
				parsed_time = Date.parse(master_timer_time);

			// if the time right now is less
			if (current_time <= parsed_time && master_timer_time != '' && master_timer_time != null && parsed_time != null) {
				return true;
			}

			return false;

		},

		add_time: function(duration = 15) {

			// go grab the current time from the master_timer variable in chrome storage
			chrome.storage.sync.get('master_timer', function(result) {

				// parse the value we have stored in chrome
				var master_timer = Date.parse(result.master_timer),
					current_time = new Date,
					new_time;

				// if there IS NO VALUE for the master timer, then simply add 15 minutes to the current time
				if (master_timer == '') {
					new_time = current_time.addMinutes(duration);
				};

				// if IT IS valid, then check to see if it is in the future or past
				if (master_timer != '') {

					// if it's in the past, then the new time should just be current time + 15 minutes
					if (master_timer < current_time) {
						new_time = current_time.addMinutes(duration);
					}

					// if it's in the future, then the new time should be the old timer + 15 minutes
					if (master_timer >= current_time) {
						new_time = master_timer.addMinutes(duration);
					}

				}

				// set the value in the chrome API and update the view
				chrome.storage.sync.set({
					master_timer: new_time.toGMTString()	  	
			  	}, function() {
			  		chrome.runtime.sendMessage({request: 'update_settings'});
			  		core.controller.update_view();
			  	});

			});	

		},

		unpause: function() {

			chrome.storage.sync.set({
				master_timer: ''
		  	}, function() {
		  		chrome.runtime.sendMessage({request: 'update_settings'});
		  		core.controller.update_view();
		  	});

		}

	},

	initialize: function() {

		core.controller.update_view();
		core.interaction.initialize();

	}

};