function blocklist_frame() {

	var frame = $("#blocklist_frame"),
		main_input = $("#blocklist-frame input#main-input"),
		blocklist_frame_msg_1 = $("#blocklist-frame #msg-1"), // "Hello there"
		blocklist_frame_msg_2 = $("#blocklist-frame #msg-2"), // "Let's become more mindful"
		blocklist_frame_msg_3 = $("#blocklist-frame #msg-3"), // "What websites woud you like to block?"
		next_button = $("#blocklist-frame #next-button-blocklist-frame"), // to go to next frame
		error_text = $("#blocklist-frame .error-text"); // this is the element that will contain any errors and let the user know why it's an error

	function starting_animation() {

		$("#blocklist-frame").delay(500) // wait half a second before starting the animation	
			.queue(function(next){ // show the first message, which is actually the 2nd message (I didn't like the first one)
				blocklist_frame_msg_2.addClass("show");
				next();
			})
			.delay(default_transition_time)
			.queue(function(next){ // show the 3rd message, which is "which websites would you like to block"
				blocklist_frame_msg_3.addClass("show");
				next();
			})
			.delay(default_transition_time)
			.queue(function(next){ // show the main input
				main_input.addClass("show").removeAttr("disabled");
				next();
			})
			.delay(1000)
			.queue(function(next) { // do the typed.js thing 

				var typed = new Typed("#typed-text", {
					strings: ["YouTube.com", "Facebook.com", "Twitter.com", "reddit.com", "Pinterest.com", "Instagram.com"],
					typeSpeed: 120,
					backSpeed: 80,
					startDelay: 0,
					loop: true,
					showCursor: true,
					cursorChar: '|',
					autoInsertCss: true
				});

				// when the main input is clicked, destroy typed.js
				main_input.on("click", function() {
					typed.destroy();
					main_input.removeClass("fresh");
				});

				// when the main input has a keypress, destroy typed.js
				main_input.on("keypress", function(){
					typed.destroy();
					main_input.removeClass("fresh");
				});

				next();
			
			});
		
	};

	// this is the format of the HTML in which things should be added to the <ul id="active-blocklist"></ul>
	// <li>facebook.com<i class="material-icons">highlight_off</i></li>
	function add_blocklist_dom(url) {

		// reset the input
		$("#blocklist-frame #main-input").val("");

		// remove any error text
		$("#blocklist-frame .error-text").css("opacity", 0);

		// this is where new entries will go
		var active_blocklist = $("#blocklist-frame ul#active-blocklist");

		// create a new element
		var li = $('<li></li>')
			.text(url)
			.attr("class", "blocklist-item")
			.data("url", url)
			.appendTo(active_blocklist, {
				css: {
					display: 'inline-block',
					opacity: 0
				}
			});

		li.animate({"opacity": 0}, 0);
		li.animate({"opacity": 1}, 1500);

		var remove = $('<i></i>')
			.attr("class", "material-icons")
			.html("highlight_off")
			.appendTo(li);

		// on the click of the "remove" icon, have the list item disappear from the DOM
		remove.on("click", function() {

			li.animate({"opacity": 0}, 1000, "swing", function(){
				li.remove();	
			});

		});

	};

	// on press of enter
	function interaction() {

		// when the user presses "enter"
		main_input.on("keypress", function(e){if (e.which == 13) {

			// grab the URL from the user and trim it
			var url = helpers.trim_url($(main_input).val());

			// validate the URL using the method in helpers.js
			var valid_url = helpers.validate_url(url);

			// if the URL is valid...
			if (valid_url.val == true) {
				
				next_button.removeClass("disabled");

				// ONLY ON VALID URLTW
				// show the "next" button if it isn't already there
				if (next_button.css("opacity").toString() != "1") {
					next_button.css("opacity", 1);
				}

				add_blocklist_dom(url);

			}

			// if the URL is not valid...
			if (valid_url.val == false) {

				// if there is already an error msg up there, but we need to change it, then go ahead and do so with an animation
				if (error_text.text().length > 0 && error_text.text() != valid_url.msg) {

					error_text.css("opacity", 0) // change the opacity to "0"
						.delay(500)
						.queue(function(next){
							
							error_text
								.css("opacity", 1)
								.html(valid_url.msg);

							next();
						
						});

				} else { // brand new error message, so don't "flash" the object

					error_text.css("opacity", 1)
						.html(valid_url.msg);
				
				}

			}

		}});

		// on the click of the "next" button
		next_button.on("click", function(){

			// if the button is not disabled (the user shouldn't be able to click it if it's opacity is 0)
			if (next_button.hasClass("disabled") != true) {

				// update the actual blocklist in the chrome API
				var user_blocklist = $(".blocklist-item").map(function(){return $(this).data("url"); }).toArray();
				chrome.storage.sync.set({blocklist: user_blocklist}, function() {
					chrome.runtime.sendMessage({request: 'update_settings'});
				});
	  
				$("#blocklist-frame").animate({"opacity": 0}, 1000, "swing", function(){
					
					$("#blocklist-frame").remove();

					email_frame();

				});
				
			}

		});

	};

	function run() {
		starting_animation();
		interaction();
	};

	run();

};