var container = {
	
	elements: {
		edit_blocklist: $("#edit-blocklist.container"),
		schedule: $("#schedule.container"),
		edit_blocklist_toggle: $('.container-toggle[data-id="edit-blocklist"]'),
		schedule_toggle: $('.container-toggle[data-id="schedule"]')
	},

	interaction: function() {

		$(".container-toggle").on("click", function() {

			if ($(this).hasClass("active") == false) {

				var prev_active_id = $(".container-toggle.active").attr("data-id"),	
					prev_active_container = $(".container").filter(function(){return $(this).attr("id") == prev_active_id;});

				$(".container-toggle").removeClass("active");		
				$(this).addClass("active");

				var animation_duration = 500;

				var active_id = $(this).attr("data-id"),
					active_container = $(".container#" + active_id);

				$(".container").css("display", "block");
				active_container.fadeOut(0);

				prev_active_container.fadeOut(animation_duration, function(){
					active_container.fadeIn(animation_duration);
				});

			}

		});
	
	},

	// right now, all this function is going to do is listen for a URL arg to be clicked
	initialize: function() {

		container.interaction();

		var url = new URL(window.location.href),
			arg = url.searchParams.get("action");

		if (arg == "schedule") {
			container.elements.schedule_toggle.click();			
		}

	}

}

// this will setup the blocklist and allow it to be manipulated locally
// the reason you have to do this is because of async within JS and the chrome API (fucking christ)
var blocklist;

// because of how the chrome.storage API works, you'll need to nest each instance where you grab data from the storage API within the chrome.storage.sync.get CALLBACK FUNCTION
// due to this, I'm creating a simple helper method that will take the data value from "result.blocklist" and format it properly if it's null or undefined
var get_blocklist = function(callback = function(){}) {
	
	var format_blocklist = function(blocklist) {

		if (typeof blocklist === "object") {
			return blocklist;
		}

		return [];

	};
	
	chrome.storage.sync.get('blocklist', function(result) {
		
		blocklist = format_blocklist(result.blocklist);

		// if there are no elements to show on the blocklist, unhide the "empty-blocklist" area
		if (blocklist.length == 0) {
			$(".empty-blocklist").show();
		} else {
			$(".empty-blocklist").hide();
		};

		callback();
	
	});

};

var sync_blocklist = function(callback = function(){}) {

	chrome.storage.sync.set({
    	// this assumes you can provide a URL for storage
    	blocklist: blocklist
  	}, function() {
    	
    	// Update status to let user know options were saved.
    	get_blocklist(callback());
    	chrome.runtime.sendMessage({request: 'update_settings'});

    	if (blocklist.length == 0) {
			$(".empty-blocklist").show();
		} else {
			$(".empty-blocklist").hide();
		};

  	});

}

var edit_blocklist = {

	elements: {
		input: $("input#add-to-blocklist")
	},

	controller: {

		add_to_blocklist: function(url) {

			// check to see if it's a valid URL
			// will return...
			// {val: true/false, msg: "Error Message"}
			is_valid = helpers.validate_url(url);
			// console.log(is_valid);
			if (is_valid.val) {

				// grab the actual URL and trim it
				url = helpers.trim_url(url);

				blocklist.unshift(url);

				// add that URL to the blocklist and sync values
				// the reason you need to pass "sync: true" is because sometimes you will want to add_to_blocklist and NOT sync - eg: when you are booting up the GUI and just want to add the DOM elements
				sync_blocklist(function(){
					edit_blocklist.view.add_to_blocklist(url);
				});

			} else {
				M.toast({html: is_valid.msg});
			};

		},

		remove_from_blocklist: function(url) {

			blocklist = blocklist.filter(function(d){return d != url});

			sync_blocklist(function(){
				edit_blocklist.view.remove_from_blocklist(url);
			});

		}

	},

	view: {

		add_to_blocklist: function(url, options = {}) {

			// this is the master element - it has the important stuff - other stuff will be added to this (favicon, remove option, etc.)
			var chip = $('<div></div>')
				.attr("class", "chip blocklist-item")
				.attr("data-url", url)
				.text(url);

			// favicon <img>
			$('<img>')
				.attr("src", "http://www.google.com/s2/favicons?domain=" + url)
				.appendTo(chip);

			// this is the remove_cirlce <i> element itself - it's just there to provide the user something to click on basically
			var close_btn = $('<i>close</i>')
				.attr("class", "material-icons")
				.appendTo(chip);

			// this will make it so that when the secondary_content is clicked you execute the remove_from_blocklist function
			close_btn.on("click", function(){
				edit_blocklist.controller.remove_from_blocklist(url);
			});

			chip.appendTo("#blocklist-area").fadeOut(0, function(){
				chip.fadeIn(1000);
			});

			// this will simply clear out the input box and remove focus from it
			edit_blocklist.elements.input.val("");

		},

		// remove the chip from the view
		remove_from_blocklist: function(url) {

			$('.blocklist-item[data-url="' + url + '"]').fadeOut(400, function(){
				$('.blocklist-item[data-url="' + url + '"]').remove();
			});

		},

		interaction: function() {
			
			// listen to keypresses on the add to blocklist thing
			edit_blocklist.elements.input.on("keypress", function(e) {

				// when you press enter do stuff
				if (e.which == 13) {

					var user_input = edit_blocklist.elements.input.val();
					edit_blocklist.controller.add_to_blocklist(user_input);

				};

			});

		},

		initialize: function() {

			// this will be for the "typed.js" user input thing
			var typed_js_input = function() {

				var options = {
					strings: ["YouTube.com", "Facebook.com", "Twitter.com", "Reddit.com", "Pinterest.com", "Instagram.com"],
					typeSpeed: 120,
					backSpeed: 80,
					startDelay: 20,
					loop: true,
					showCursor: true,
					cursorChar: '|',
					autoInsertCss: true
				};

				var typed = new Typed("#typed-text", options);

				edit_blocklist.elements.input.on("click", function(){
					typed.destroy();
					edit_blocklist.elements.input.removeClass("fresh");
				});

			}();

			// get the user's current blocklist from the chrome storage API
			blocklist = get_blocklist(function() {

				// if the user doesn't have a blocklist setup AND has not onboarded, then redirect them to the onboarding proces
				// this will ONLY redirect if the user has not previously gone through the onboarding process
				if (blocklist.length == 0) {
					
					var onboard = chrome.extension.getURL('onboard.html');
					chrome.storage.sync.get("onboarded", function(result) {

						var onboarded = result.onboarded; 
						if (typeof onboarded === 'undefined') {onboarded = 'false';};
							
						if (onboarded.toString() != "true") {
							window.location.href = onboard;
						};

					});

				}

				// add all existing blocklist items to the DOM
				for (var i = blocklist.length - 1; i >= 0; i--) {
					edit_blocklist.view.add_to_blocklist(blocklist[i], {sync: false});
				};

			});

			edit_blocklist.view.interaction();

		}

	},


};

// this will be for the porn blocker checkbox
var adult_filter = {

	// on change of the adult filter checkbox
	listener: function(){

		$("input#adult-filter").on("change", function(){

			var checked = $(this).is(":checked").toString();

			chrome.storage.sync.set({
		    	adult_filter: checked
		  	}, function() {
		    	chrome.runtime.sendMessage({request: 'update_settings'});
		  	});			

		});

	},

	initialize: function() {

		// read to see if the master switch is on/off and adjust the DOM accordingly 
		chrome.storage.sync.get('adult_filter', function(result) {
			
			var adult_filter = result.adult_filter;
			
			if (adult_filter == "true") {
				$("input#adult-filter").prop("checked", true);
			}

		});

		adult_filter.listener();

	}

};

var schedule_mode = {

	controller: {
		
		save: function() {

			console.log('save');

			// parse timepickers
			function parse() {

				var schedule = {
					times: {},
					days: {}
				};

				schedule.times.to = $('#to-timepicker.timepicker').val();
				schedule.times.from = $('#from-timepicker.timepicker').val();

				$('#repeat-form input[type="checkbox"]').each(function(){
					
					var day = $(this).val(),
						is_active = $(this).prop("checked");

					schedule.days[day] = is_active;

				});

				return schedule;

			};

			// we just have one schedule for now, but wrap it in an array so we can add more later if we need to
			var schedules = [parse()];

			chrome.storage.sync.set({
				schedules: schedules
			}, function() {
				chrome.runtime.sendMessage({request: 'update_settings'});
				schedule_mode.view.save();
			});

		}

	},

	view: {

		save: function() {

			console.log("view save");
			
			M.toast({html: 'Schedule saved!'});

			// if schedule mode is turned off, turn it on when you save a new schedule
			if ($("#schedule-mode-switch").prop("checked") == false) {
				$("#schedule-mode-switch").click();
			}

			core.controller.update_view();

		},

		update: function() {

			// this will change the sidebar feature ("On" or "Off") as well as the master switch
			function update_on_off_ui(is_on) {

				if (is_on == true) {
					$("#schedule-mode-badge").addClass("green").removeClass("red").attr("data-badge-caption", "On");
					$("#schedule-mode-switch").prop("checked", true);
				} else {
					$("#schedule-mode-badge").removeClass("green").addClass("red").attr("data-badge-caption", "Off");
					$("#schedule-mode-switch").prop("checked", false);
				}

			};

			// this will make it so that the view the user sees actually matches the schedule they've previously set 
			function update_schedule_user_settings_ui(schedules) {

				// later on we'll implement more than one schedule, but for now this will suffice
				var schedule = schedules[0];

				// set default times based on users current schedule
				var to_time = schedule.times.to,
					from_time = schedule.times.from;
				
				$('#from-timepicker.timepicker').val(from_time);
				$("#to-timepicker.timepicker").val(to_time);

				$.each(schedule.days, function(k, v){
					$('#repeat-form input[value="' + k + '"]').prop("checked", v);
				});

			};

			chrome.storage.sync.get(["schedule_mode_master_switch", "schedules"], function(result) {

				var schedule_mode_master_switch = result.schedule_mode_master_switch; 
				if (typeof schedule_mode_master_switch === 'undefined') {schedule_mode_master_switch = 'false';};
					
				if (schedule_mode_master_switch.toString() == "true") {
					update_on_off_ui(true);
				} else {
					update_on_off_ui(false);
				};

				var schedules = result.schedules;

				update_schedule_user_settings_ui(schedules);

			});

		},

		interaction: function() {

			$("#schedule-save-button").on("click", function() {
				console.log('$("#schedule-save-button")');
				schedule_mode.controller.save();
			});

			$("#schedule-mode-switch").on("click", function(){
				
				var is_on = $(this).prop("checked").toString();

				// set dat value in chrome boi
			  	chrome.storage.sync.set({

			    	// this assumes you can provide a URL for storage
			    	schedule_mode_master_switch: is_on
			  	
			  	}, function() {

			    	schedule_mode.view.update();
			    	chrome.runtime.sendMessage({request: 'update_settings'});

			  	});

			});

			$("#edit-my-schedule-btn").on("click", function(){
				$('li.container-toggle[data-id="schedule"]').click();
			});

		}

	},

	initialize: function() {

		// initialize the timepickers to work
		$('#from-timepicker.timepicker').timepicker();
		$('#to-timepicker.timepicker').timepicker();

		schedule_mode.view.interaction();
		schedule_mode.view.update();

	}

};

$(document).ready(function(){

	container.initialize();
	edit_blocklist.view.initialize();
	schedule_mode.initialize();
	adult_filter.initialize();
	// adult_filter.listener();
	
	core.initialize();

});